{
***************************************************************
* Cdigo para la aplicacin del algoritmo de Cohen-Sutherland *
* Escrito por Nicols Rozas Salgado                           *
***************************************************************

El siguiente cdigo es un ejemplo de la aplicacin del algoritmo de
recortado de lneas de "Cohen-Sutherland". El area de recorte est
definido en la constante "Recorte" y en el grfico que se muestra
se puede observar un rectngulo verde que lo representa.
En color blanco aparece la linea original y luego en rojo aparece la
linea ya recortada por medio del algoritmo.

Nota:
* Para dibujar las lneas se aplic el algoritmo de Bresenham.

}
Uses Crt,Dos;
Type Rect = Record
            X1,Y1,X2,Y2: Real;
            End;
Const
Abajo = 1;
Arriba = 2;
Derecha = 4;
Izquierda = 8;
Recorte: Rect = (X1: 25; Y1: 25; X2: 75; Y2: 75); {El area de recorte}

Elementos = 10; {Cantidad de elementos maximos a agregar}
Var
Cantidad: Byte;
Puntos: Array[1..Elementos] Of Rect;

{Agrega una nueva serie de puntos al array}
Procedure Agregar(X0,Y0,X1,Y1: Real);
Begin
Inc(Cantidad);
If Cantidad <= Elementos Then
 Begin
 Puntos[Cantidad].X1:= X0;
 Puntos[Cantidad].Y1:= Y0;
 Puntos[Cantidad].X2:= X1;
 Puntos[Cantidad].Y2:= Y1;
 End;
End;

Function CalcularCodigo(X,Y: Real; R: Rect): Integer;
Var Res: Integer;
Begin
Res:= 0;
If (Y > R.Y2) Then Inc(Res,Abajo) Else
If (Y < R.Y1) Then Inc(Res,Arriba);
If (X > R.X2) Then Inc(Res,Derecha) Else
If (X < R.X1) Then Inc(Res,Izquierda);
CalcularCodigo:= Res;
End;

Function Cortar(Var X0,Y0,X1,Y1: Real; Re: Rect): Boolean;
Var X,Y: Real;
C0,C1,C: Integer;
Aceptar,Hecho: Boolean;
Begin
Aceptar:= False;
Hecho:= False;
C0:= CalcularCodigo(X0,Y0,Re);
C1:= CalcularCodigo(X1,Y1,Re);
GotoXY(1,23);
Write('Entrada a "Cohen-Sutherland": ');
GotoXY(1,24);
Repeat
If ((C0 Or C1) = 0) Then
 Begin
 {Aceptacion trivial: Ambos terminan en el rectangulo}
 Write('Ambos adentro');
 Aceptar:= True; Hecho:= True;
 End Else
If ((C0 And C1) <> 0) Then
 Begin
 {Rechazo trivial: Ambos terminan del lado externo del rectangulo}
 Write('Ambos afuera');
 Hecho:= True;
 End Else
Begin
 {Un caso normal: Recortar lo que este afuera del rectangulo}
 If C0 <> 0 Then C:= C0 Else C:= C1;
 If (C And Abajo) <> 0 Then
  Begin
  X:= X0 + (X1 - X0) * (Re.Y2 - y0) / (y1 - y0);
  y:= Re.y2;
  Write('Abajo, ');
  End Else
 If (C And Arriba) <> 0 Then
  Begin
  X:= X0 + (X1 - X0) * (Re.Y1 - y0) / (y1 - y0);
  y:= Re.Y1;
  Write('Arriba, ');
  End Else
 If (C And Derecha) <> 0 Then
  Begin
  X:= Re.X2;
  Y:= Y0 + (Y1 - Y0) * (Re.X2 - X0) / (X1 - X0);
  Write('Derecha, ');
  End Else
  Begin
  X:= Re.X1;
  Y:= Y0 + (Y1 - Y0) * (Re.X1 - X0) / (X1 - X0);
  Write('Izquierda, ');
  End;

{Establece el nuevo punto final e itera}
If (C = C0) Then
 Begin
 x0:= x; y0:= y;
 C0:= CalcularCodigo(x0, y0, Re);
 End Else
  Begin
  x1:= x; y1:= y;
  C1:= CalcularCodigo(x1, y1, Re);
  End;
End;
Until Hecho;
Cortar:= Aceptar;
End;

{A partir de aqui esta seccion esta dedicada a la muestra de graficos
en pantalla}
Procedure ModoPantalla(Valor: Byte);
Var Regs: Registers;
Begin
Regs.Ax:= Valor;
Intr($10,Regs);
End;

Procedure Graficos; Begin ModoPantalla($13); End;

Procedure Texto; Begin ModoPantalla(3); End;

Procedure Plot(X,Y: Word; Color: Byte);
Begin
Mem[SegA000:(Y*320)+X]:= Color;
End;

Procedure LimpiarPantalla;
Begin
FillChar(Mem[SegA000:0],64000,0);
End;

Procedure linea(x,y,x2,y2,col:integer);
var d,dx,dy,ai,bi,xi,yi:integer;
begin
 if(x<x2)then begin xi:=1;dx:=x2-x;end else begin xi:=-1;dx:=x-x2;end;
 if(y<y2)then begin yi:=1;dy:=y2-y;end else begin yi:=-1;dy:=y-y2;end;
 plot(x,y,col);
 if dx>dy then begin ai:=(dy-dx)*2;bi:=dy*2; d:=bi-dx;
  repeat
   if(d>=0)then begin inc(y,yi);inc(d,ai);end else inc(d,bi);
   inc(x,xi);plot(x,y,col);
   until(x=x2);
  end
 else begin ai:=(dx-dy)*2;bi:=dx*2; d:=bi-dy;
  repeat
   if(d>=0)then begin inc(x,xi);inc(d,ai);end else inc(d,bi);
   inc(y,yi);plot(x,y,col);
   until(y=y2);
  end;
 end;

Procedure Rectangulo;
Var I,J: Word;
Begin
For J:= Trunc(Recorte.Y1) To Trunc(Recorte.Y2) Do
For I:= Trunc(Recorte.X1) To Trunc(Recorte.X2) Do
 Plot(I,J,2);
End;

{Seccion principal del programa}
Var Pos: Byte;
X1,Y1,X2,Y2: Real;
Begin
Cantidad:= 0;

{Agrega aqui mismo los puntos que desees probar}
Agregar(15,50,30,60);
Agregar(50,15,50,30);
Agregar(50,15,40,32);
Agregar(50,55,50,90);
Agregar(50,55,40,90);
Agregar(85,12,73,47);
Agregar(85,12,85,27);
Agregar(70,50,100,50);
Agregar(86,70,86,80);
Agregar(88,74,86,80);

Graficos;
DirectVideo:= False; {Para poder usar Write y WriteLn en modo graficos}
Pos:= 1;
While Pos <= Cantidad Do
 Begin
 LimpiarPantalla;
 Rectangulo;
 X1:= Puntos[Pos].X1;  Y1:= Puntos[Pos].Y1;
 X2:= Puntos[Pos].X2;  Y2:= Puntos[Pos].Y2;
 GotoXY(1,19);
 Write('Linea ',Pos);
 GotoXY(1,20);
 Write('Original: (',Trunc(X1),',',Trunc(Y1),') (',Trunc(X2),',',Trunc(Y2),')');
 Linea(Trunc(X1),Trunc(Y1),Trunc(X2),Trunc(Y2),15);
 If Cortar(X1,Y1,X2,Y2,Recorte) Then
  Begin
  Linea(Trunc(X1),Trunc(Y1),Trunc(X2),Trunc(Y2),4);
  GotoXY(1,21);
  Write('Recortada: (',Trunc(X1),',',Trunc(Y1),') (',Trunc(X2),',',Trunc(Y2),')');
  End;
 ReadKey;
 Inc(Pos);
 End;
Texto;
End.
